import json
import logging
import sys
from pathlib import Path

import deqart

from .exceptions import DeqartBaseException

logger = logging.getLogger("deqart-python-sdk")


def ask_token():
    config_dir = Path.home() / ".config" / "deqart"
    config_filename = "config.json"
    config_file = config_dir / config_filename
    if config_file.is_file():
        yes_no = input(f"File {config_file} exists. Do you want to overwrite? [y/n] : ")
        if yes_no != "y":
            return
    token = input("Input the SDK token deqart.com : ")
    config_dir.mkdir(exist_ok=True, parents=True)
    if config_file.is_file():
        existing_config = json.load(open(config_file))
        existing_config["token"] = token
        json.dump(existing_config, open(config_file, "w"), indent=4)
        logger.info("Configuration file %s successfully updated.", config_file)
    else:
        json.dump(
            {
                "token": token,
                "main_endpoint": "https://firebasetestserver-aclbgnvcha-uc.a.run.app",
                "ssl_verify": True,
            },
            open(config_file, "w"),
            indent=4,
        )
        logger.info("Configuration file %s successfully created.", config_file)


def main():
    available_commands = "Available commands to deqart CLI are: init version"
    if len(sys.argv) == 1:
        raise DeqartBaseException(
            0, "No command given to deqart CLI. " + available_commands
        )
    command = sys.argv[1]

    if command == "init":
        ask_token()
    elif command == "version":
        print(f"Deqart Python SDK version {deqart.__version__}")
    else:
        raise DeqartBaseException(
            0,
            "Wrong command " + command + " to deqart CLI. " + available_commands,
        )


if __name__ == "__main__":
    main()



import datetime
import logging
import time

import dateutil.parser

from ..api import API
from ..exceptions import DeqartBaseException
from ..lib import circuit_serialization

logger = logging.getLogger("deqart-python-sdk")

_api = API.get_instance()


def search_jobs(run_status=None, limit=100, created_later_than=None):
    if created_later_than is None:
        parsed_created_later_than = None
    elif isinstance(created_later_than, str):
        parsed_created_later_than = dateutil.parser.parse(created_later_than)
        if parsed_created_later_than.tzinfo is None:
            logger.warning(
                "created_later_than is a str object without timezone info, assuming UTC timezone"
            )
            parsed_created_later_than = parsed_created_later_than.replace(
                tzinfo=datetime.timezone.utc
            )
        parsed_created_later_than = parsed_created_later_than.isoformat()
    elif isinstance(created_later_than, datetime.datetime):
        if created_later_than.tzinfo is None:
            raise DeqartBaseException(
                0, "created_later_than is a datetime object without timezone info"
            )
        parsed_created_later_than = created_later_than.isoformat()
    else:
        raise DeqartBaseException(
            0, "created_later_than should be None, str, or datetime.datetime object"
        )

    params = {
        "limit": limit,
        "run_status": run_status,
        "created_later_than": parsed_created_later_than,
    }
    result_dict = {"column_names": [], "data": []}
    while True:
        response = _api.send_request(req_type="GET", path="/jobs", params=params)
        if not response.ok:
            raise DeqartBaseException(
                response.status_code, "Couldn't search jobs " + response.text
            )
        response = response.json()

        result_dict["column_names"] = response["column_names"]
        results = response["data"]
        result_dict["data"] += results
        result_dict["total_count"] = response["total_count"]

        if len(results) < limit:
            break

        params["offset"] = len(result_dict["data"])
    return result_dict


def estimate_job_runtime(circuit):
    encoded_circuit = circuit_serialization.encode_circuit(circuit)
    params = encoded_circuit
    params["estimate_only"] = True
    response = _api.send_request(req_type="POST", path="/jobs", json_req=params)
    if not response.ok:
        raise DeqartBaseException(
            response.status_code, "Couldn't estimate job " + response.text
        )
    return response.json()


def submit_job(circuit):
    encoded_circuit = circuit_serialization.encode_circuit(circuit)
    params = {"circuit": encoded_circuit}
    response = _api.send_request(req_type="POST", path="/jobs", json_req=params)
    if not response.ok:
        raise DeqartBaseException(
            response.status_code, "Couldn't submit job " + response.text
        )
    return response.json()["data"]


def wait_for_job(job_id):
    while True:
        response = _api.send_request(req_type="GET", path=f"/jobs/{job_id}")
        if not response.ok:
            raise DeqartBaseException(
                response.status_code, "Couldn't search jobs " + response.text
            )
        response = response.json()["data"]
        if response["run_status"] != "COMPLETED":
            time.sleep(10.0)
        else:
            return response



import logging
from pathlib import Path

import packaging.version
import requests

from .api import API as _API
from .db.jobs import estimate_job_runtime, search_jobs, submit_job, wait_for_job
from .version import __version__

formatter = logging.Formatter(fmt="DEQART-PYTHON-SDK - %(levelname)s - %(message)s")
# formatter = logging.Formatter(fmt='%(asctime)s - %(levelname)s - %(module)s - %(message)s')

handler = logging.StreamHandler()
handler.setFormatter(formatter)

logger = logging.getLogger("deqart-python-sdk")
logger.setLevel(logging.INFO)
logger.addHandler(handler)


def _check_version():
    local_version = packaging.version.parse(__version__)
    if local_version.is_prerelease:
        logger.warning(
            "Development version %s of Deqart Python SDK is being used", __version__
        )
    req = requests.get("https://pypi.python.org/pypi/deqart/json", timeout=5.0)
    if not req.ok:
        return

    # find max version on PyPI
    releases = req.json().get("releases", [])
    pip_version = packaging.version.parse("0")
    for release in releases:
        ver = packaging.version.parse(release)
        if not ver.is_prerelease or local_version.is_prerelease:
            pip_version = max(pip_version, ver)

    if pip_version.major > local_version.major:
        logger.warning(
            "There is a major upgrade of Deqart Python SDK available on PyPI. We recommend upgrading. Run 'pip install --upgrade deqart' to upgrade from your version %s to %s.",
            local_version,
            pip_version,
        )
    elif pip_version > local_version:
        logger.info(
            "There is a newer version of Deqart Python SDK available on PyPI. Run 'pip install --upgrade deqart' to upgrade from your version %s to %s.",
            local_version,
            pip_version,
        )


def init(
    api_token: str | None = None, path_to_config_json: str | Path | None = None
) -> _API:
    """Initializes and authenticates to Deqart platform using the config file.
    If not initialized then $HOME/.config/deqart/config.json
    will be used.

    :param api_token: The API token
    :type api_token: str | None

    :param path_to_config_json: Location to config JSON file
    :type path_to_config_json: str | Path | None
    """
    _api = _API.get_instance()
    _api.init(api_token, config_location=path_to_config_json)
    return _api


_check_version()
